/* ------------------------------------------------------------- */
/* TVID: Three-loop Vacuum Integrals from Dispersion relations   */
/* version 1.0 (Dec 16, 2016)                                    */
/* ------------------------------------------------------------- */
/* ucall.cc                                                      */
/* Ayres Freitas (afreitas@pitt.edu)			         */
/* last revision 13.12.16                                        */
/* ------------------------------------------------------------- */
/* Main program for calling C(++) routines for evaluation of     */
/* finite parts of U4, U5, U6 integrals                	         */
/* ------------------------------------------------------------- */
/* Reference:                                                    */
/*   [1] = Freitas, arXiv:1609.09159, JHEP 1611 (2016) 145       */
/* ------------------------------------------------------------- */

#include <iostream>
#include <fstream>
#include <stdio.h>
#include <string.h>
using namespace std;
	
#include "Cplx.h"
#include "U456.h"


#ifdef QUADPRECISION
Double Cprec=1e-12;
#else
Double Cprec=1e-6;
#endif


int main(int argc, char *argv[])
{
/*  Cplx res;
  Double abserr;
  Double m1s = SDouble("1e-6"),
	 m2s = SDouble("1"),
	 m3s = SDouble("0.7"),
	 m4s = SDouble("1e-6"),
	 m5s = SDouble("0.73469387755102040816326530612245"),
	 m6s = SDouble("6");
  
  cout.precision(16);
  res = U40int(m2s,m3s,m4s,Cprec,&abserr);
//  res = U5int(m1s,m2s,m3s,m4s,m5s,Cprec,&abserr);
//  res = U6(m1s,m2s,m3s,m4s,m5s,m6s,Cprec,&abserr);
  cout << res << "\t" << abserr << endl;
  res = U4int(m1s,m2s,m3s,m4s,Cprec,&abserr);
  cout << res << "\t" << abserr << endl;
*/
  if(argc < 3)
  {
    cerr << "Error: no input/output file specified" << endl;
    return 1;
  }
  
  char line[500], *strp;
  Double pars[9];
  unsigned pc;
  Double abserr;
  Double ires;
  
  ifstream inf(argv[1], ios::in );
  if(!inf)
  {
    cerr << "Error in input file " << argv[1] << endl;
    exit(1);
  }
  ofstream ouf(argv[2], ios::out );
  if(!ouf)
  {
    cerr << "Error in output file " << argv[2] << endl;
    exit(1);
  }
  ouf.precision(16);
//  cout.precision(30);

  while(inf.getline(line, sizeof(line)-1))
  {
    strp = strtok(line, " \t");
    cout << strp << endl;
    strp = strtok(NULL, " \t");
    for(pc = 0; pc < sizeof(line) && strp != NULL; pc++)
    {
      pars[pc] = atodd(strp);
      strp = strtok(NULL, " \t");
    }
    if(strp != NULL)
      cerr << "Too many parameters in\n " << line << endl;
    strp = strtok(line, " \t");

    if(strcmp(strp, "U4") == 0 && pc == 4)
      ires = U4int(pars[0], pars[1], pars[2], pars[3], Cprec, &abserr);
    if(strcmp(strp, "U40") == 0 && pc == 3)
      ires = U40int(pars[0], pars[1], pars[2], Cprec*10, &abserr);
    if(strcmp(strp, "U5") == 0 && pc == 5)
      ires = U5int(pars[0], pars[1], pars[2], pars[3], pars[4], Cprec, &abserr);
    if(strcmp(strp, "U50") == 0 && pc == 3)
      ires = U50int(pars[0], pars[1], pars[2], Cprec, &abserr);
    if(strcmp(strp, "U6") == 0 && pc == 6)
      ires = U6int(pars[0], pars[1], pars[2], pars[3], pars[4], pars[5],
                   Cprec, &abserr);
    if(abs(abserr/ires) > 1e-10)
      cerr << "Warning: precision of result may be less than 10 digits! "
       << abs(abserr/ires) << "\n";
    ouf << ires << endl;
  }
  inf.close();
  ouf.close();

  return 0;
}
